/*!
 * @file network_stats_widget.h
 * @brief Definition of an EnergyWidget, that displays a bar and a Text
 */

#ifndef _NETWORK_STATS_WIDGET_H
#define _NETWORK_STATS_WIDGET_H

#include "glgui_fixedposition_box.h"
#include "glgui_bar.h"
#include "glgui_text.h"
#include "glgui_pushbutton.h"

#include "network/ip.h"

class NetworkMonitor;
class NetworkNode;


class HostWidget : public OrxGui::GLGuiBox
{
  public:
    HostWidget(const std::string& name, const IP& ip);

    void setName(const std::string& name) { this->_name.setText(name); };
    void setIP(const IP& ip) { this->_ip.setText(ip.ipString()); this->_storedIP = ip; };

    void setNameWidth(float width) { this->_name.setLineWidth(width); };

    bool operator==(const IP& ip) const { return (this->_storedIP == ip); };
    bool operator==(const std::string& name) const { return (this->_name == name); };

  protected:
    virtual void showing();
    virtual void hiding();

  private:
    OrxGui::GLGuiText _name;           //!< The Name of the Proxy server to be displayed.
    OrxGui::GLGuiText _ip;             //!< The IP of the proxy server.
    IP                _storedIP;       //!< The ip to compare.
};


class NodeWidget : public OrxGui::GLGuiBox
{
  public:
    NodeWidget(const std::string& proxyName, const IP& ip);
    NodeWidget(const NetworkNode* node);

    void addClient(const NetworkNode* node);
    void addClient(const std::string& name, const IP& ip);

    bool removeClient(const IP& ip);
    bool removeClient(const std::string& name);
    bool removeClient(const std::string& name, const IP& ip);

    void setClientNameWidths(float width);


  protected:
    virtual void hiding();
    virtual void showing();


  private:
    HostWidget                _proxyWidget;

    std::vector<HostWidget*>  _clients;
    float                     _clientNameWidth;
};




//! A class to display network Statistics.
class NetworkStatsWidget : public OrxGui::GLGuiFixedpositionBox
{
  public:
    static void toggleGUI();

    NetworkStatsWidget(const NetworkMonitor* monitor);
    virtual ~NetworkStatsWidget();

    void setUpstream(unsigned int upstream);
    void setDownstream(unsigned int upstream);
    void setIP(const IP& ip);



    void addProxy(const std::string& name, const IP& proxy);
    void addNode(const NetworkNode* node);

    void clearProxies();

    void rebuild();

    //void rebuildConnectedHosts(std::vector<hosts> hosts);

    void setMaximum(float max);
    void setValue(float value);

    virtual void tick(float dt);

  protected:
    virtual void resize();
    virtual void showing();
    virtual void hiding();

  private:
    const NetworkMonitor*  _monitor;

    OrxGui::GLGuiText      _thisHostIs;
    HostWidget             _thisHost;

    OrxGui::GLGuiText      _upstreamText;
    OrxGui::GLGuiText      _downstreamText;

    OrxGui::GLGuiWidget    _serverBox;

    std::vector<NodeWidget*> _proxies;


    OrxGui::GLGuiPushButton _refreshButton;

    static NetworkStatsWidget*    _statsWidget;

    //OrxGui::GLGuiText       _valueText;
    //OrxGui::GLGuiBar        _bar;
};

#endif /* _NETWORK_STATS_WIDGET_H */
